<?php

namespace Drupal\commerce_ticketing\Plugin\QueueWorker;

use Drupal\commerce_ticketing\Mail\TicketReceiptMail;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_ticketing\CommerceTicketInterface;

/**
 * Processes send mail tasks for commerce_ticketing receipt.
 *
 * @QueueWorker(
 *   id = "commerce_ticketing_send_ticket_receipt_worker",
 *   title = @Translation("Commerce Ticketing send ticket receipt worker"),
 *   cron = {"time" = 30}
 * )
 */
class CommerceTicketingSendTicketReceiptWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The order_item storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $orderItemStorage;

  /**
   * The ticket receipt mail.
   *
   * @var \Drupal\commerce_ticketing\Mail\TicketReceiptMail
   */
  protected $ticketReceiptMail;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, TicketReceiptMail $ticket_receipt_mail, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->orderItemStorage = $entity_type_manager->getStorage('commerce_order_item');
    $this->ticketReceiptMail = $ticket_receipt_mail;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('commerce_ticketing.ticket_receipt_mail'),
      $container->get('logger.factory')->get('commerce_ticketing')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if ($data instanceof CommerceTicketInterface) {
      $order_item = $this->orderItemStorage->load($data->get('order_item_id')->getValue()[0]['target_id']);
      $result = $this->ticketReceiptMail->send($data);
      if (!$result) {
        $error_message = $this->t('Ticket email could not be sent.');

        // This will ensure that the item stays in the queue and gets
        // processed later again.
        throw new \Exception($error_message);
      }
      else {
        $ticket_mail_sent_order_item_data = $order_item->getData('ticket_mail_sent');
        $ticket_mail_sent_order_item_data[$data->id()] = TRUE;

        $order_item->setData('ticket_mail_sent', $ticket_mail_sent_order_item_data);
        $order_item->save();
        $this->logger->info('Mail sent for ticket @number.', ['@number' => $data->getTicketNumber()]);
      }
    }
  }

}
